<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Framework\App\Config\ScopeConfigInterface;

/**
 * Class AbstractExporter
 * @package FiloBlu\Refilo\Model\Exporter
 */
abstract class AbstractExporter
{
    /**
     * @var Mongo
     */
    private $mongo;
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * AbstractExporter constructor.
     * @param Mongo $mongo
     */
    public function __construct(Mongo $mongo, ScopeConfigInterface $scopeConfig)
    {
        $this->mongo = $mongo;
        $this->scopeConfig = $scopeConfig;
    }

    public function export()
    {
        $this->preExecute();
        $this->execute();
        $this->postExecute();
    }

    public function preExecute()
    {
        foreach ($this->getCollections() as $collection) {
            $this->mongo->dropCollection($collection, $this->getDatabase());
        }
    }

    abstract public function getCollections() : iterable;

    /**
     * @return mixed
     */
    public function getDatabase()
    {
        $database = $this->scopeConfig->getValue('mongo/general/mongo_general_database');

        if (!$database) {
            throw new \RuntimeException('MongoDB is not configured');
        }

        return $database;
    }

    abstract public function execute();

    /**
     *
     */
    public function postExecute()
    {
        $this->getConnection()->createWildIndex($this->getCollections());
    }

    /**
     * @return Mongo
     */
    public function getConnection(){
        return $this->mongo;
    }
}
